'use strict';

Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const ali_oss_1 = tslib_1.__importDefault(require("ali-oss"));
const publisher_base_1 = tslib_1.__importDefault(require("@electron-forge/publisher-base"));
const async_ora_1 = require("@electron-forge/async-ora");
class PublisherOss extends publisher_base_1.default {
    constructor() {
        super(...arguments);
        this.name = 'oss';
    }
    async publish({ makeResults }) {
        const { config } = this;
        const ossClient = new ali_oss_1.default(config);
        const artifacts = [];
        for (const makeResult of makeResults) {
            artifacts.push(...makeResult.artifacts.map((artifact) => ({
                path: artifact,
                version: makeResult.packageJSON.version,
                platform: makeResult.platform,
                arch: makeResult.arch,
            })));
        }
        let uploaded = 0;
        let releaseArtifact;
        const details = [];
        const spinnerText = () => `Uploading Artifacts ${uploaded}/${artifacts.length}\n  ${details.join('\t')}`;
        await (0, async_ora_1.asyncOra)(spinnerText(), async (uploadSpinner) => {
            await Promise.all(artifacts.map(async (artifact, i) => {
                const name = this.generateName(artifact);
                const artifactPath = artifact.path;
                const basename = path_1.default.basename(artifactPath);
                const extname = path_1.default.extname(artifactPath);
                await ossClient.multipartUpload(name, artifactPath, {
                    progress: (p) => {
                        details[i] = `<${basename}>: ${Math.round(p * 100)}%`;
                        uploadSpinner.text = spinnerText();
                    }
                });
                uploaded += 1;
                details[i] = `<${basename}>: 100%`;
                uploadSpinner.text = spinnerText();
                if (extname && ['.dmg', '.exe'].includes(extname)) {
                    releaseArtifact = artifact;
                }
            }));
        });
        if (releaseArtifact) {
            this.setRelease(ossClient, releaseArtifact);
        }
    }
    generateName(artifact) {
        const { config: { basePath = '' } } = this;
        const { platform, version, path: artifactPath } = artifact;
        return `${basePath}/${platform}/${version}/${path_1.default.basename(artifactPath)}`;
    }
    async setRelease(ossClient, artifact) {
        const { config: { basePath = '' } } = this;
        const { platform, version, path: artifactPath } = artifact;
        let releaseJson;
        try {
            let result = await ossClient.get(`${basePath}/${platform}/release.json`);
            releaseJson = JSON.parse(`${result.content}`);
        }
        catch (e) {
            releaseJson = {
                currentRelease: '',
                releases: []
            };
        }
        const url = ossClient.generateObjectUrl(`${basePath}/${platform}/${version}/${encodeURIComponent(path_1.default.basename(artifactPath))}`);
        releaseJson.currentRelease = version;
        releaseJson.releases.push({
            version,
            updateTo: {
                version,
                pub_date: new Date(),
                notes: `version：${version}`,
                name: version,
                url
            }
        });
        const result = await ossClient.put(`${basePath}/${platform}/release.json`, Buffer.from(JSON.stringify(releaseJson)));
        if (result === null || result === void 0 ? void 0 : result.name) {
            console.log('  <release.json> uploaded successfully!');
        }
    }
}
exports.default = PublisherOss;
